/* -----------------------------------------------------------------------------------------

                   SASEFRED Macro Library

        Name: fred_filetransurl.sas  
 Description: This macro reads FRED data by FILENAME trans URL access.
              The SASEFRED Interface Engine                   
           Title: Reading FRED Data by URL Access on FILENAME Statement.
     Product: SAS/ETS Software                                    
        Keys: FRED data extraction, XML FRED Data                             
       Procs: SASEFRED
       Notes: This is an uncompiled macro, used by SASEFRED to build     
                  a FRED request for web access. An API_KEY provided by 
                  FRED is required:

                  filename trans url
                     'https://api.stlouisfed.org/fred/series/observations?'
                         seriesid='cfnai'&freq='d'&sort='asc'
                            units='lin'&agg='avg'&
                               start=YYYYMMDD&end=YYYYMMDD&
                                  limit= 100000&offset=0&
                                     out_type=1&file_type=xml 
                  api_key ='0932f481a9ddeaa52778ea8706ac13ee';

              30nov2012, saskff pulled for 13.1 copy at v940m1,
                         using s0923282.
              23may2014, saskff pulled for 14.1 copy at v940m3,
                         using s1035706.
              30Mar2015, saskff pulled for 14.1 v940m3 to add proxy= option for SAS/Germany 
                         using s1159906.
              20Jul2015, saskff pulled to update 15.1 to https: instead of http:
                         using S1190360. Also added connect= option per Eric Thies suggestion.

     Support: saskff

 Required Parameters: 
 
  apikey    - string, FRED web service required key, 
              32 character alpha-numeric lowercase string.
  ids       - string, BY groups or ids that the time series are organized.
  

 Additional optional parameters follow:
  freq      - string, a lower frequency to aggregate values to, default: no value.
              for no frequency aggregation. Options: 'd','w','bw','m','q','sa',
              'a','wef','weth','wew','wetu','wem','bwew','bwem'.
  sort      - string, Sort results is ascending or descending order, default: 'asc'. 
  units     - string, key that indicates a data value transformation, default: 'lin'.
  agg       - string, the aggregation method used for frequency aggregation,
              default: avg. Options: 'avg','sum','eop'.
  start     - string, start of the real-time period, default: today's date.
  end       - string, end of the real-time period, default: today's date.
  vintage   - string, used to download data as it existed on these specified dates in history.
              they are used in conjunction with output=3 or output=4.
  limit     - integer, maximum number of results to return, default: 100000.
  offset    - integer, non-negative, default: 0.
  output    - integer, default: 1. Options: '1','2','3','4'.
  format    - string, format, so far we use XML only. 
  dbug      - string, flag to get more debug info in the SAS log about the macro
              variables and their values. Uses PROC Printto to redirect the log
              information just like makefile.sas macro in dvtest/testauto does.
  numIds    - integer, number that indicates the index of the time series that is
              passed to the macro. E.g., if idlist ='firstSeries, secondSeries', the 
              numIds corresponding to firstSeries is 1, and the 
              numIds corresponding to secondSeries is 2.
  rtstart   - string, real-time start of the time period, only used if rtflag is set.
  rtend     - string, real-time end of the time period, only used if rtflag is set.
  rtflag    - integer, flag set when user wants to use a real-time period, usually
              when output = 4 for Initial release only, open it wide to get everything.
  proxy     - use a proxy server at this address
  proxyflag - default is zero, set to one when proxy is specified.
  connect   - Yes or No to connect through firewall on PROXY, s1190360, hot fix.

 
------------------------------------------------------------------------------------------*/
%macro fred_filetransurl( apikey=,ids=,freq=,sort=,units=,agg=,
                         start=,end=,limit=,offset=,output=,vintage=,
                         format=,dbug=,numIds=,rtstart=,rtend=,rtflag=,
                         proxy=,proxyflag=0,connect=OFF );

    /*--- preset status to bad for early returns ---*/
    %let fred_macro_status=1;
    %let debug=&dbug;
    %let realtime=&rtflag;

    %if %sysfunc(upcase(&debug))=ON %then %do;
       options mprint symbolgen;
    %end;
    %else %do;
       options nonotes;        /*  s1111072, unresolved macros...*/
                               /*  %filename(newlog,newlog,tmp); */
       filename newlog "newlog.tmp";
       proc printto log=newlog new;
       run;
       options notes;
    %end;
    %let zero = %str(0);
    %let connflag=&connect;
    %let globproxy=OFF;

    %if &connflag=ON %then %do;
        %let myconnect = %str(connect);
    %end;
    %else %do;
        %let myconnect= ;
    %end;

    %if &proxyflag EQ &zero %then %do;
       %let proxy_len=0;
    %end;
    %else %do;
        %let proxy_len = %eval( %length(&proxy) );
        %let myproxy = %str(proxy=);
        
        %if &proxy_len > 2 %then %let globproxy = ON;
        
        %else %let proxy_len = 0;
    %end;

    options validvarname=any; 
    %let myurl     = %str(https://api.stlouisfed.org/fred/series/observations?);
    %let myid      = %str(series_id=);
    %let myapi     = %nrstr(&api_key=);
    %let myfreq    = %nrstr(&frequency=);
    %let mysort    = %nrstr(&sort_order=);
    %let myunit    = %nrstr(&units=);
    %let myagg     = %nrstr(&aggregation_method=);
    %let mystart   = %nrstr(&observation_start=);
    %let myend     = %nrstr(&observation_end=);
    %let myvin     = %nrstr(&vintage_dates=);
    %let mylmt     = %nrstr(&limit=);
    %let myoff     = %nrstr(&offset=);
    %let myout     = %nrstr(&output_type=);
    %let myform    = %nrstr(&file_type=);
    %let myrtstart = %nrstr(&realtime_start=);
    %let myrtend   = %nrstr(&realtime_end=);


    %let nrbid     = %str(&ids);
    %let nrbapi    = %str(&apikey);
    %let nrbfreq   = %str(&freq); 
    %let nrbsort   = %str(&sort);
    %let nrbunit   = %str(&units);
    %let nrbagg    = %str(&agg);    
    %let nrbstart  = %str(&start);
    %let nrbend    = %str(&end);
    %let nrbvin    = %str(&vintage);
    %let nrblmt    = %str(&limit);
    %let nrboff    = %str(&offset);
    %let nrbout    = %str(&output);
    %let nrbform   = %str(&format);

    %if (&realtime=1) %then %do;
       %let nrbrtstart=%str(&rtstart);
       %let nrbrtend=%str(&rtend);
 /*--- remove the leading and trailing quotes from the rtstart (&realtime_start=) ---*/
       %let rtstartlen = %eval( %length(&rtstart)-2 );   
       %if &rtstartlen < 0 %then %goto skiprt7;
       %if &rtstartlen = 0 %then %do;       
          %let nrbrtstart =; 
          %goto skiprt7;
       %end;  
       %let nrbrtstart = %substr( &rtstart, 2 , &rtstartlen );

   %skiprt7:
 /*--- remove the leading and trailing quotes from the realtime_end (&realtime_end=) ---*/
       %let rtendlen = %eval( %length(&rtend)-2 );   
       %if &rtendlen < 0 %then %goto skiprt8;
       %if &rtendlen = 0 %then %do;       
          %let nrbrtend =; 
          %goto skiprt8;
       %end;  
       %let nrbrtend = %substr( &rtend, 2 , &rtendlen );

   %skiprt8:

    %end; /* end &realtime=1 */

   %skip0:
 /*--- remove the leading and trailing quotes from the apikey (&api_key=) ---*/
    %let apilen = %eval( %length(&apikey)-2 );
    %if &apilen < 0 %then %goto skip1;
    %if &apilen = 0 %then %do;          /*to avoid warning message when the &apilen=0*/
       %let nrbapi =; 
       %goto skip1;
    %end;      
    %let nrbapi = %substr( &apikey, 2 , &apilen );
   
   %skip1:            
 /*--- remove the leading and trailing quotes from the time series ID (ids) (&seriesid=) ---*/
    %let idlen = %eval( %length(&ids)-2 );
    %if &idlen < 0 %then %goto skip2;
    %if &idlen = 0 %then %do;       
       %let nrbid =; 
       %goto skip2;
    %end;            
    %let nrbid = %substr( &ids, 2 , &idlen );

   %skip2:
 /*--- remove the leading and trailing quotes from (&frequency=) ---*/
    %let freqlen = %eval( %length(&freq)-2 );
    %if &freqlen < 0 %then %goto skip3;
    %if &freqlen = 0 %then %do;       
       %let nrbfreq =; 
       %goto skip3;
    %end;       
    %let nrbfreq = %substr( &freq, 2 , &freqlen );

   %skip3:
 /*--- remove the leading and trailing quotes from the sort (&sort=) ---*/
    %let sortlen = %eval( %length(&sort)-2 );
    %if &sortlen < 0 %then %goto skip4;
    %if &sortlen = 0 %then %do;       
       %let nrbsort =; 
       %goto skip4;
    %end;     
    %let nrbsort = %substr( &sort, 2 , &sortlen );

   %skip4:
 /*--- remove the leading and trailing quotes from the units (&units=) ---*/
    %let unitlen = %eval( %length(&units)-2 );
    %if &unitlen < 0 %then %goto skip5;
    %if &unitlen = 0 %then %do;       
       %let nrbunit =; 
       %goto skip5;
    %end;  
    %let nrbunit = %substr( &units, 2 , &unitlen );

   %skip5:
 /*--- remove the leading and trailing quotes from the aggregation_method (&aggregation_method=) ---*/
    %let agglen = %eval( %length(&agg)-2 );   
    %if &agglen < 0 %then %goto skip6;
    %if &agglen = 0 %then %do;       
       %let nrbagg =; 
       %goto skip6;
    %end;  
    %let nrbagg = %substr( &agg, 2 , &agglen );

   %skip6:
 /*--- remove the leading and trailing quotes from the observation_start (&observation_start=) ---*/
    %let startlen = %eval( %length(&start)-2 );   
    %if &startlen < 0 %then %goto skip7;
    %if &startlen = 0 %then %do;       
       %let nrbstart =; 
       %goto skip7;
    %end;  
    %let nrbstart = %substr( &start, 2 , &startlen );

   %skip7:
 /*--- remove the leading and trailing quotes from the observation_end (&observation_end=) ---*/
    %let endlen = %eval( %length(&end)-2 );   
    %if &endlen < 0 %then %goto skip8;
    %if &endlen = 0 %then %do;       
       %let nrbend =; 
       %goto skip8;
    %end;  
    %let nrbend = %substr( &end, 2 , &endlen );

   %skip8:
/*--- remove the leading and trailing quotes from the vintage_date (&vintage_date=) ---*/
    %let vinlen = %eval( %length(&vintage)-2 );   
    %if &vinlen < 0 %then %goto skip9;
    %if &vinlen = 0 %then %do;       
       %let nrbvin =; 
       %goto skip9;
    %end;  
    %let nrbvin = %substr( &vintage, 2 , &vinlen );
   
   %skip9:
 /*--------------------------------------------------------*/
 /*--- Validation of parameters supplied to macro       ---*/
 /*--------------------------------------------------------*/  
    %if &nrbapi EQ %then %do;
        %put ERROR: API_KEY= parameter required for FRED_FILETRANSURL macro.;
        %goto exit;
    %end;

 /*--------------------------------------------------------*/
 /*--- Build FILENAME statement with URL for FRED       ---*/
 /*--------------------------------------------------------*/
    %if &nrbid= %then %do; /* do1 */
        filename trans&numIds url
        'https://api.stlouisfed.org/fred/series/observations?series_id=cfnai&format=xml&myapi.&nrbapi';

    %goto exit;
    %end;  /*end of do1 */

    /* -------- requests that have option VINTAGE =     ------------------------- */
    %if &vinlen <=0 %then %goto skip_vin;

    /* -------- if output_type = 3 or 4 then UNITS=lin  ------------------------- */
    %if (&nrbout=3) or (&nrbout=4) %then %do; /* do2 */
       /* -- UNITS= is set to 'lin' in the following request        -----*/
       %if (&realtime=1) %then %do;
          %let glob = &myurl.&myid.&nrbid.&myapi.&nrbapi.&myform.&nrbform.&myvin.&nrbvin.&mysort.&nrbsort.&mylmt.&nrblmt.&myoff.&nrboff.&myunit.lin&myfreq.&nrbfreq.&myagg.&nrbagg.&myout.&nrbout.&myrtstart.&nrbrtstart.&myrtend.&nrbrtend;
       
          %if &globproxy=OFF %then %do;
              filename trans&numIds url "&glob"
              &myconnect;
          %end;
          %else %do;                                          
              %if &connect=ON %then %do; 
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy"
                 connect;
              %end;
              %else %do;
                  filename trans&numIds url "&glob"
                  &myproxy "&proxy";
              %end;
          %end;

       %end;
       %else %do;
          %let glob = &myurl.&myid.&nrbid.&myapi.&nrbapi.&myform.&nrbform.&myvin.&nrbvin.&mysort.&nrbsort.&mylmt.&nrblmt.&myoff.&nrboff.&myunit.lin&myfreq.&nrbfreq.&myagg.&nrbagg.&myout.&nrbout;
      
          %if &globproxy=OFF %then %do;
              filename trans&numIds url "&glob"
              &myconnect;
          %end;
          %else %do;
              %if &connect=ON %then %do; 
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy"
                 connect;
              %end;
              %else %do;
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy";
              %end;
          %end;

       %end;
    %end;  /*end of do2 */

    /* -------- if output_yype = 1 or 2 then UNITS= can assume any valid value -- */
    %else %if (&nrbout=1) or (&nrbout=2) %then %do;  /* do3 */
       %if (&realtime=1) %then %do;
          %let glob = 
  &myurl.&myid.&nrbid.&myapi.&nrbapi.&myform.&nrbform.&mystart.&nrbstart.&myend.&nrbend.&myvin.&nrbvin.&mysort.&nrbsort.&mylmt.&nrblmt.&myoff.&nrboff.&myunit.&nrbunit.&myfreq.&nrbfreq.&myagg.&nrbagg.&myout.&nrbout.&myrtstart.&nrbrtstart.&myrtend.&nrbrtend


;
          %if &globproxy=OFF %then %do;
              filename trans&numIds url "&glob"
              &myconnect;
          %end;
          %else %do;
              %if &connect=ON %then %do;                              
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy"
                 connect;
              %end;
              %else %do;
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy";
              %end;
          %end;

       %end;    
       %else %do;
          %let glob = &myurl.&myid.&nrbid.&myapi.&nrbapi.&myform.&nrbform.&mystart.&nrbstart.&myend.&nrbend.&myvin.&nrbvin.&mysort.&nrbsort.&mylmt.&nrblmt.&myoff.&nrboff.&myunit.&nrbunit.&myfreq.&nrbfreq.&myagg.&nrbagg.&myout.&nrbout;
       
          %if &globproxy=OFF %then %do;
              filename trans&numIds url "&glob"
              &myconnect;
          %end;
          %else %do;
              %if &connect=ON %then %do;
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy"
                 connect;
              %end;
              %else %do;
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy";
              %end;
          %end;

       %end;
    %end;  /*end of do3 */
     
    %else %do;  /* do4 */
   %skip_vin:
       %if (&realtime=1) %then %do;
          %let glob = 
  &myurl.&myid.&nrbid.&myapi.&nrbapi.&myform.&nrbform.&mystart.&nrbstart.&myend.&nrbend.&mysort.&nrbsort.&mylmt.&nrblmt.&myoff.&nrboff.&myunit.&nrbunit.&myfreq.&nrbfreq.&myagg.&nrbagg.&myout.&nrbout.&myrtstart.&nrbrtstart.&myrtend.&nrbrtend;
       
          %if &globproxy=OFF %then %do;
              filename trans&numIds url "&glob"
              &myconnect;
          %end;
          %else %do;
              %if &connect=ON %then %do;
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy"
                 connect;
              %end;
              %else %do;
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy";
              %end;
          %end;

       %end;
       %else %do;
          %let glob = &myurl.&myid.&nrbid.&myapi.&nrbapi.&myform.&nrbform.&mystart.&nrbstart.&myend.&nrbend.&mysort.&nrbsort.&mylmt.&nrblmt.&myoff.&nrboff.&myunit.&nrbunit.&myfreq.&nrbfreq.&myagg.&nrbagg.&myout.&nrbout;
      
          %if &globproxy=OFF %then %do;
              filename trans&numIds url "&glob"
              &myconnect;
          %end;
          %else %do;
              %if &connect=ON %then %do;
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy"
                 connect;
              %end;
              %else %do;
                 filename trans&numIds url "&glob"
                 &myproxy "&proxy";
              %end;
          %end;

       %end;
    %end; /*end of do4 */

  %exit:
    /* -- unconditionally cut notes on --; */
    options nonotes; 
    
    %if %sysfunc(upcase(&debug))=OFF %then %do;
       proc printto log=log;
       run;

    /* -- s1111072, unresolved macros...*/
    /* -- filename newlog "newlog.tmp"; */
    /* -- %ridfile(newlog.tmp);         */

      %let rc = %sysfunc(fdelete(newlog));
    %end; 

    options notes; options validvarname=any; 

    %let fred_macro_status=0;
%mend fred_filetransurl;
