/* -----------------------------------------------------------------------------------------

                   SASEFRED Macro Library

        Name: fred_freqcheck.sas  
 Description: This macro submits a request to the FRED server and obtains the attrobutes of an 
              economic timeseries. Then, it compares the default frequency of the timeseries 
              with the requestd frequency and returns error message if the default frequency is 
              lower than the requested FREQ.                  
       Title: Reading FRED Data by URL Access on FILENAME Statement.
     Product: SAS/ETS Software                                    
        Keys: FRED data extraction, XML FRED Data                             
       Procs: SASEFRED
       Notes: This is an uncompiled macro, used by SASEFRED to build     
                  a FRED request for web access. An API_KEY provided by 
                  FRED is required:

                  filename trans url
                     'https://api.stlouisfed.org/fred/series?
                         seriesid=gnpca&api_key =0932f481a9ddeaa52778ea8706ac13ee;'

              30nov2012, saskff pulled for 13.1 copy at v940m1,
                         using s0923282.
              24apr2014, saskff pulled for 13.2 to add seriesAvailDate
                         at v940m2, using s1030196.
              24Mar2015, saskff pulled for 14.1 v940m3 to add proxy= option for SAS/Germany 
                         using s1159906.
              20Jul2015, saskff pulled to update 15.1 to https: instead of http:
                         using S1190360. Also added connect= option per Eric Thies suggestion.


     Support: saskff

 Required Parameters: 
 
  apikey    - string, FRED web service required key, 
               32 character alpha-numeric lowercase string.
  idsf      - string, BY groups or ids that the time series are organized.
  freqf     - string, a lower frequency to aggregate values to, default: no value.
              for no frequency aggregation. Options: 'd','w','bw','m','q','sa',
              'a','wef','weth','wew','wetu','wem','bwew','bwem'.
  dbugf     - string, flag to get more debug info in the SAS log about the macro.
              variables and their values. Uses PROC Printto to redirect the log
              information just like makefile.sas macro in dvtest/testauto does.
  automapf  - replace or reuse or no, when NO use access=READONLY.
  mapreff   - filename handle (MyMaptp) when automap=NO access=READONLY.                   
  xmlmapf   - full physical pathname and filename of the xmlmap. 
  outXmlf   - full physical path to filename of the XML file.
  numIdsf   - integer, number that indicates the index of the time series that is
              passed to the macro. E.g., if idlist ='firstSeries, secondSeries', the 
              numIds corresponding to firstSeries is 1, and the 
              numIds corresponding to secondSeries is 2.

  proxyf     - use a proxy server at this address
  proxyflagf - default is zero, set to one when proxyf is specified.
  connectf   - Yes or No to connect through firewall on PROXY, s1190360, hot fix.

  MacroStructure: 
              %macro fred_freqcheck();  Macro 1
                 %macro fred_filetransurl_f(...); Macro 1.1
                    .............
                 %mend fred_filetransurl_f; 
                    
                 %macro fred_filegetxml_f(...);   Macro 1.2
                    .............
                 %mend fred_filegetxml_f;
              
                 %macro fred_filemapxml_f(...);   Macro 1.3
                    ..............
                    %macro compareFreq;          Calling Macro 2
                 %mend fred_filemapxml_f;
                     
                 %fred_filetransurl_f;   Calling Macro 1.1
                 %fred_filegetxml_f;     Calling Macro 1.2
                 %fred_filemapxml_f;     Calling Macro 1.3
              %mend fred_freqcheck();

              %macro compareFreq(...); Macro 2
                 .............. 
              %mend compareFreq;
------------------------------------------------------------------------------------------*/


/*Macro 1 --------------------------------------------------------------------------------*/

%macro fred_freqcheck( apikeyf=,idsf=,freqf=,outXmlf=,automapf=,mapreff=,xmlmapf=,
                       numIdsf=,dbugf=,proxyf=,proxyflagf=0,connectf=OFF );

%let debug=&dbugf;

%if %sysfunc(upcase(&debug))=ON %then %do;
   options mprint symbolgen;
%end;
%else %do;
   options nonotes;        /*  s1111072, unresolved macros...*/
                           /*  %filename(newlog,newlog,tmp); */
   filename newlog "newlog.tmp";
   proc printto log=newlog new;
   run;
   options notes;
%end;

%let zero = %str(0);
%let connflag=&connectf;
%let globproxy=OFF;

%if &connflag=ON %then %do;
    %let myconnect = %str(connect);
%end;
%else %do;
    %let myconnect= ;
%end;

%if &proxyflagf EQ &zero %then %do;
   %let proxy_len = 0;
%end;
%else %do;
    %let proxy_len = %eval( %length(&proxyf) );
    %let myproxy = %str(proxy=);
    
    %if &proxy_len > 2 %then %let globproxy = ON;
    
    %else %let proxy_len = 0;
%end;

/* SubMacro 1.1 --------------------------------------------------------------------------*/
%macro fred_filetransurl_f( apikey=,ids=,numIds=,proxy=,proxyflag=0,connect=OFF );

    /*--- preset status to bad for early returns ---*/
    %let fred_macro_status=1;

    options validvarname=any; 
    %let myurl     = %str(https://api.stlouisfed.org/fred/series?);
    %let myid      = %str(series_id=);
    %let myapi     = %nrstr(&api_key=);
    
    %let nrbid     = %str(&ids);
    %let nrbapi    = %str(&apikey);

    %let zero = %str(0);


  
 /*--- remove the leading and trailing quotes from the apikey (&api_key=) ---*/
    %let apilen = %eval( %length(&apikey)-2 );
    %if &apilen < 0 %then %goto skip1;
    %if &apilen = 0 %then %do;          /*to avoid warning message when the &apilen=0*/
       %let nrbapi =; 
       %goto skip1;
    %end;      
    %let nrbapi = %substr( &apikey, 2 , &apilen );
   
   %skip1:            
 /*--- remove the leading and trailing quotes from the time series ID (ids) (&seriesid=)--*/
    %let idlen = %eval( %length(&ids)-2 );
    %if &idlen < 0 %then %goto skip2;
    %if &idlen = 0 %then %do;       
       %let nrbid =; 
       %goto skip2;
    %end;            
    %let nrbid = %substr( &ids, 2 , &idlen );

  %skip2:
 /*--------------------------------------------------------*/
 /*--- Validation of parameters supplied to macro       ---*/
 /*--------------------------------------------------------*/  
    %if &nrbapi EQ %then %do;
        %put ERROR: API_KEY= parameter required for FRED_FILETRANSURL macro.;
        %goto exit;
    %end;

 /*--------------------------------------------------------*/
 /*--- Build FILENAME statement with URL for FRED       ---*/
 /*--------------------------------------------------------*/
    %if &nrbid= %then %do;
       %if &globproxy=OFF %then

        filename transtp&numIds url
   'https://api.stlouisfed.org/fred/series/search?search_text=money%20stock'                
        &myapi.&nrbapi
        &myconnect;

       %else %do;
           %if &connect=ON %then %do; 

        filename transtp&numIds url
   'https://api.stlouisfed.org/fred/series/search?search_text=money%20stock'                
        &myapi.&nrbapi &myproxy "&proxy"
        connect;
           %end;

           %else %do;
        filename transtp&numIds url
   'https://api.stlouisfed.org/fred/series/search?search_text=money%20stock'                
        &myapi.&nrbapi &myproxy "&proxy";
           %end;

       %end; /* globproxy=on */
    %end; /* nrbid=NULL */
    %else %do;
       %let glob = &myurl.&myid.&nrbid.&myapi.&nrbapi;
       %if &globproxy=OFF %then
          filename transtp&numIds url "&glob"
          &myconnect;
       %else %do;
           %if &connect=ON %then %do;
              filename transtp&numIds url "&glob"
              &myproxy "&proxy"
              connect;
           %end;
           %else %do;
              filename transtp&numIds url "&glob"
              &myproxy "&proxy";
           %end;
       %end; /* globproxy=on */
    %end; /* nrbid not null */

%exit:
    %let fred_macro_status=0;

%mend fred_filetransurl_f;

 
/* SubMacro 1.2 --------------------------------------------------------------------------*/

%macro fred_filegetxml_f(outXml=,numIds=);

    /*--- preset status to bad for early returns ---*/
    %let fred_macro_status=1;
    %let fred_xmlout_exist=1;

    options validvarname=any;

    /*--------------------------------------------------------*/
    /*--- Validation of parameters supplied to macro       ---*/
    /*--------------------------------------------------------*/

    %let nrbxout = %str(&&outXml.tp&numIds);
    %let ext = %str(xml);

    %if %eval(%length(&nrbxout) ne 0) %then %do;
       %let glob = &nrbxout..&ext;
       filename outtp&numIds "&glob";

       data _NULL_;
       FILE outtp&numIds;
       INFILE transtp&numIds; /*  must match fred_filetransurl's fileref  */
       INPUT;
       PUT _infile_;
       RUN;

    %end;
    %else %do;
       filename outtp&numIds "XFRED.xml";

       data _NULL_;
       FILE outtp&numIds;
       INFILE transtp&numIds; /*  must match fred_filetransurl's fileref */
       INPUT;
       PUT _infile_;
       RUN;
   
    %end;
  
    %let fred_outxml_exist = %sysfunc(FILEREF(outtp&numIds));
    %put NOTE: fred_outxml_exist flag is set to &fred_outxml_exist;

    %if %sysfunc(FILEREF(outtp&numIds)) ne 0 %then %do;
        %put %sysfunc(sysmsg());
        %goto bad_status; /* return bad status since no XML file for XMLV2 engine */
    %end;

    %let fred_macro_status=0;
    %bad_status:

%mend fred_filegetxml_f;


/*SubMacro 1.3----------------------------------------------------------------------------*/

%macro fred_filemapxml_f( automap=,mapref=,xmlmap=,outXml=,freq=,numIds=);

    /*-----------------------------------------------------------------------*/
    /*-- fasayy: squotelst macro  to put ' ' around the elements of a list --*/
    /*-----------------------------------------------------------------------*/   
    %macro squotelst(strng,quote1=%str(%'),quote2=%str(%'),delim=%str( ));
        %local i squotelst;
        %let i=1;
        %do %while(%length(%qscan(&strng,&i,|)) GT 0);
            %if %length(&squotelst) EQ 0 %then %let squotelst=&quote1.%qscan(&strng,&i,|)&quote2;
            %else %let squotelst=&squotelst.&quote1.%qscan(&strng,&i,|)&quote2;
            %let i=%eval(&i + 1);
        %if %length(%qscan(&strng,&i,|)) GT 0 %then %let squotelst=&squotelst.&delim;
        %end;
        %unquote(&squotelst)
    %mend squotelst;
    
    /*--- preset status to bad for early returns ---*/
    %let fred_macro_status=1;
    %let fred_xmlout_exist=1;

/*--- remove the leading and trailing quotes from the freq (&freq=) ---*/
    %let frqlen = %eval( %length(&freq)-2 );

/*  NEED TO CONSIDER THE CASE WHERE FREQ IS NULL 
    %if &freqlen < 0 %then %goto skip1;
    %if &apilen = 0 %then %do;        
       %let nrbapi =; 
       %goto skip1;
    %end;
*/  
    
    %let nrbfreq = %substr( &freq, 2 , &frqlen );
  
    %let myxmlmap  = %str(xmlmap=);
    %let myaccess  = %str(access=READONLY);
    %let mylib     = %str(lib=outtp);
    %let myautomap = %str(automap=);

    %let nrbauto   = %upcase(&automap);
    %let nrbmapref = %str(&&mapref.tp&numIds);

    %let len = %length(&xmlmap);
    %let nrbxmap = %substr( &xmlmap, 2 , %eval(&len-2) );
    %let part1 = %substr(&nrbxmap, 1 , %eval(&len-6) );
    %let mapName=%sysfunc(scan(&part1,-1,'\'));
    %let lenMap=%length(&mapName);
    %let part1 = %substr(&nrbxmap, 1 , %eval(&len-6-%eval(&lenMap)) );
    %let part2 = %substr(&nrbxmap, &len-4 , %eval(3) );

    %if %sysfunc(upcase(&part2))=MAP %then %do; /* the file extension is map and it is correct */
       %let nrbxmap = &part1.XFREDTP&numIds..&part2;
       %let nrbxmap = %squotelst(&nrbxmap);
       %put &nrbxmap;
    %end;
    %else %do; /* the file extension is not correct */
       %put ERROR: the xmlmap extension (map) need to be specified. ;
       %goto fatal_return;
    %end;

    /*--------------------------------------------------------*/
    /*--- Validation of parameters supplied to macro       ---*/
    /*--------------------------------------------------------*/
    %if &nrbxmap EQ %then %do;
        %put ERROR: XMLMAP= parameter is required for FRED_FILEMAPXML macro.;
        %goto exit;
    %end;

    %let freq_match_flag=1;  /* reset the flag (failure) */

    /*-----------------------------------------------------------*/
    /*--- Issue XMLV2 LIBNAME assign based on automap setting ---*/
    /*--- Use the same handle name as the XML fileref......   ---*/
    /*-----------------------------------------------------------*/
    %if &nrbauto=NO %then %do;

        filename &nrbmapref "&nrbxmap";
        /* libname outxml xmlv2 xmlmap=mapref access=READONLY */
        libname outtp&numIds xmlv2 &myxmlmap.&nrbmapref &myaccess;
        proc datasets &mylib&numIds;
        run;
        proc contents data=outtp&numIds..SERIES; run;
        proc print data=outtp&numIds..SERIES; run;
        proc contents data=outtp&numIds..SERIESS; run;
        proc print data=outtp&numIds..SERIESS; run;

        data _null_;
           set outtp&numIds..series;
           call symput('seriesFreq',series_frequency_short);
           call symput('seriesAvailDate',series_observation_start);
           
        run;

        /* if the requested freq is null, use the native freq instead */
        %put here the nrbfreq is &nrbfreq;
        %if &nrbfreq EQ %then %do;                            
           %let nrbfreq=&seriesFreq;
        %end;
       
        %compareFreq(nrbfreq1=&nrbfreq,seriesFreq1=&seriesFreq);
        %put default frequency is &seriesFreq;
        %put requested frequency is &nrbfreq;
        %put freq_match_flag is &freq_match_flag; 
        %if  &freq_match_flag=1 %then %do;
           %goto fatal_return;
        %end;

        %put here the seriesAvailDate is &seriesAvailDate;
        %let fred_macro_start=&seriesAvailDate;

    %end;
    %else %do;
        %let fred_outxml_exist = %sysfunc(FILEREF(outtp&numIds));
        %put NOTE: fred_outxml_exist flag is set to &fred_outxml_exist;

        %if %sysfunc(FILEREF(outtp&numIds)) ne 0 %then %do;
            %put %sysfunc(sysmsg());
            %goto fatal_return; /* return bad status since no XML file for XMLV2 engine */
        %end;

        filename &nrbmapref &nrbxmap;
        libname outtp&numIds xmlv2 &myxmlmap.&nrbmapref &myautomap.&nrbauto;
        proc datasets &mylib&numIds;
        run;

        proc contents data=outtp&numIds..SERIES; run;
        proc print data=outtp&numIds..SERIES; run;
        proc contents data=outtp&numIds..SERIESS; run;
        proc print data=outtp&numIds..SERIESS; run;

        data _null_;
           set outtp&numIds..series;
           call symput('seriesFreq',series_frequency_short);
           call symput('seriesAvailDate',series_observation_start);
        run;

        %put here1 the nrbfreq is &nrbfreq;
        %if &nrbfreq EQ %then %do;
           %let nrbfreq=&seriesFreq;
        %end;

        %compareFreq(nrbfreq1=&nrbfreq,seriesFreq1=&seriesFreq);
        %put default frequency is &seriesFreq;
        %put requested frequency is &nrbfreq;
        %put freq_match_flag is &freq_match_flag; 
        
        %if  &freq_match_flag=1 %then %do;
           %goto fatal_return; /* failure: native freq < requested freq */
        %end;

        %put here1 the seriesAvailDate is &seriesAvailDate;
        %let fred_macro_start=&seriesAvailDate;
     
    %end;

 %exit:
    %let fred_macro_status=0;   /* SUCCESS */

 %fatal_return:

%mend fred_filemapxml_f;

/* calling SubMacros 1.1, 1.2, and 1.3 */
%fred_filetransurl_f( apikey=&apikeyf,ids=&idsf,numIds=&numIdsf,proxy=&proxyf,proxyflag=&proxyflagf,connect=&connectf);
%fred_filegetxml_f( outXml=XFRED,numIds=&numIdsf);
%fred_filemapxml_f( automap=&automapf,mapref=&mapreff,xmlmap=&xmlmapf,outXml=&outXmlf,freq=&freqf,numIds=&numIdsf);

options nonotes; 
   %if %sysfunc(upcase(&debug))=OFF %then %do;
      proc printto log=log;
      run;
    /* -- s1111072, unresolved macros...*/
    /* -- %ridfile(newlog.tmp);         */

       %let rc = %sysfunc(fdelete(newlog));
   %end; 
options notes; options validvarname=any; 

%mend fred_freqcheck;  /* end of Macro 1 */

/*Macro 2 --------------------------------------------------------------------------*/
/* Macro 2 is called inside SubMacro 1.3                                            */
/* compareFreq macro, compares the defualt frequency of the timeseries               */
/* with the requested frequency and sets freq_match_flag=0 if they                   */
/* match and sets freq_match_flag=1, otherwise.                                      */
/*-----------------------------------------------------------------------------------*/
 
%macro compareFreq(nrbfreq1=,seriesFreq1=);

   %if %sysfunc(upcase(&nrbfreq1))=D AND %sysfunc(upcase(&seriesFreq1))=D %then 
   %do;
      %let freq_match_flag=0;    /* SUCCESS */
   %end;
   %else %if (%sysfunc(upcase(&nrbfreq1))=W AND %sysfunc(upcase(&seriesFreq1))=D)    OR 
             (%sysfunc(upcase(&nrbfreq1))=W AND %sysfunc(upcase(&seriesFreq1))=W)    OR
             (%sysfunc(upcase(&nrbfreq1))=WEF AND %sysfunc(upcase(&seriesFreq1))=D)  OR 
             (%sysfunc(upcase(&nrbfreq1))=WEF AND %sysfunc(upcase(&seriesFreq1))=W)  OR
             (%sysfunc(upcase(&nrbfreq1))=WETH AND %sysfunc(upcase(&seriesFreq1))=D) OR 
             (%sysfunc(upcase(&nrbfreq1))=WETH AND %sysfunc(upcase(&seriesFreq1))=W) OR
             (%sysfunc(upcase(&nrbfreq1))=WEW AND %sysfunc(upcase(&seriesFreq1))=D)  OR 
             (%sysfunc(upcase(&nrbfreq1))=WEW AND %sysfunc(upcase(&seriesFreq1))=W)  OR
             (%sysfunc(upcase(&nrbfreq1))=WETU AND %sysfunc(upcase(&seriesFreq1))=D) OR 
             (%sysfunc(upcase(&nrbfreq1))=WETU AND %sysfunc(upcase(&seriesFreq1))=W) OR
             (%sysfunc(upcase(&nrbfreq1))=WEM AND %sysfunc(upcase(&seriesFreq1))=D)  OR 
             (%sysfunc(upcase(&nrbfreq1))=WEM AND %sysfunc(upcase(&seriesFreq1))=W)  OR
             (%sysfunc(upcase(&nrbfreq1))=WESU AND %sysfunc(upcase(&seriesFreq1))=D) OR 
             (%sysfunc(upcase(&nrbfreq1))=WESU AND %sysfunc(upcase(&seriesFreq1))=W) OR
             (%sysfunc(upcase(&nrbfreq1))=WESA AND %sysfunc(upcase(&seriesFreq1))=D) OR 
             (%sysfunc(upcase(&nrbfreq1))=WESA AND %sysfunc(upcase(&seriesFreq1))=W) %then 
           %do;
              %let freq_match_flag=0;    /* SUCCESS */
           %end;
   %else %if (%sysfunc(upcase(&nrbfreq1))=BW AND %sysfunc(upcase(&seriesFreq1))=D)    OR 
             (%sysfunc(upcase(&nrbfreq1))=BW AND %sysfunc(upcase(&seriesFreq1))=W)    OR
             (%sysfunc(upcase(&nrbfreq1))=BW AND %sysfunc(upcase(&seriesFreq1))=BW)   OR 
             (%sysfunc(upcase(&nrbfreq1))=BWEM AND %sysfunc(upcase(&seriesFreq1))=D)  OR 
             (%sysfunc(upcase(&nrbfreq1))=BWEM AND %sysfunc(upcase(&seriesFreq1))=W)  OR
             (%sysfunc(upcase(&nrbfreq1))=BWEM AND %sysfunc(upcase(&seriesFreq1))=BW) OR 
             (%sysfunc(upcase(&nrbfreq1))=BWEW AND %sysfunc(upcase(&seriesFreq1))=D)  OR 
             (%sysfunc(upcase(&nrbfreq1))=BWEW AND %sysfunc(upcase(&seriesFreq1))=W)  OR
             (%sysfunc(upcase(&nrbfreq1))=BWEW AND %sysfunc(upcase(&seriesFreq1))=BW) %then 
           %do;
              %let freq_match_flag=0;    /* SUCCESS */
           %end;
      %else %if (%sysfunc(upcase(&nrbfreq1))=M AND %sysfunc(upcase(&seriesFreq1))=D)  OR 
             (%sysfunc(upcase(&nrbfreq1))=M AND %sysfunc(upcase(&seriesFreq1))=W)  OR
             (%sysfunc(upcase(&nrbfreq1))=M AND %sysfunc(upcase(&seriesFreq1))=BW) OR
             (%sysfunc(upcase(&nrbfreq1))=M AND %sysfunc(upcase(&seriesFreq1))=M) %then 
           %do;
              %let freq_match_flag=0;    /* SUCCESS */
           %end;
   %else %if (%sysfunc(upcase(&nrbfreq1))=Q AND %sysfunc(upcase(&seriesFreq1))=D)  OR 
             (%sysfunc(upcase(&nrbfreq1))=Q AND %sysfunc(upcase(&seriesFreq1))=W)  OR
             (%sysfunc(upcase(&nrbfreq1))=Q AND %sysfunc(upcase(&seriesFreq1))=BW) OR
             (%sysfunc(upcase(&nrbfreq1))=Q AND %sysfunc(upcase(&seriesFreq1))=M)  OR
             (%sysfunc(upcase(&nrbfreq1))=Q AND %sysfunc(upcase(&seriesFreq1))=Q) %then 
           %do;
              %let freq_match_flag=0;    /* SUCCESS */
           %end;
   %else %if (%sysfunc(upcase(&nrbfreq1))=SA AND %sysfunc(upcase(&seriesFreq1))=D)  OR 
             (%sysfunc(upcase(&nrbfreq1))=SA AND %sysfunc(upcase(&seriesFreq1))=W)  OR
             (%sysfunc(upcase(&nrbfreq1))=SA AND %sysfunc(upcase(&seriesFreq1))=BW) OR
             (%sysfunc(upcase(&nrbfreq1))=SA AND %sysfunc(upcase(&seriesFreq1))=M)  OR
             (%sysfunc(upcase(&nrbfreq1))=SA AND %sysfunc(upcase(&seriesFreq1))=Q)  OR
             (%sysfunc(upcase(&nrbfreq1))=SA AND %sysfunc(upcase(&seriesFreq1))=SA) %then 
           %do;
             %let freq_match_flag=0;    /* SUCCESS */
           %end;
   %else %if (%sysfunc(upcase(&nrbfreq1))=A AND %sysfunc(upcase(&seriesFreq1))=D)  OR 
             (%sysfunc(upcase(&nrbfreq1))=A AND %sysfunc(upcase(&seriesFreq1))=W)  OR
             (%sysfunc(upcase(&nrbfreq1))=A AND %sysfunc(upcase(&seriesFreq1))=BW) OR
             (%sysfunc(upcase(&nrbfreq1))=A AND %sysfunc(upcase(&seriesFreq1))=M)  OR
             (%sysfunc(upcase(&nrbfreq1))=A AND %sysfunc(upcase(&seriesFreq1))=Q)  OR
             (%sysfunc(upcase(&nrbfreq1))=A AND %sysfunc(upcase(&seriesFreq1))=SA) OR 
             (%sysfunc(upcase(&nrbfreq1))=A AND %sysfunc(upcase(&seriesFreq1))=A) %then 
           %do;
              %let freq_match_flag=0;    /* SUCCESS */
           %end;
   %else 
      %do;
         %let freq_match_flag=1;
      %end;
%mend;
